/************************************************
* DLL.H - definitions & prototypes for the DLL	*
* function library								*
*												*
* Anthony D. Ennis								*
************************************************/

/* Debug parameters */
#define DLL_CB		1	/* Display LCB			*/
#define DLL_LINK	2	/* Display link data	*/

/* The five major points in a list. Prev & next	*/
/* are also used for DIRECTION on Add & Delete	*/
typedef enum {
	DLL_HEAD,		/* Start of list 			*/
	DLL_TAIL,		/* End of list 				*/
	DLL_CURR,		/* Current position 		*/
	DLL_NEXT,		/* Next node after current	*/
	DLL_PREV		/* Node previous to current	*/
	} Position;

typedef enum {
	DLL_SUCCESS=0,		/* Operation successful	*/
	DLL_MATCH,			/* Match found			*/
	DLL_MATCH_EOL,		/* Match at End Of List	*/
	DLL_NOMATCH,		/* No match found		*/
	DLL_BAD_NORAM=10,	/* Ran out of RAM		*/
	DLL_BAD_LIST,		/* Bad List parameter	*/
	DLL_BAD_PARAM,		/* Other bad parameter	*/
	DLL_BAD_DIRECTION,	/* Bad direction param	*/
	DLL_BAD_START,		/* Bad starting position*/
	DLL_BAD_END,		/* Bad ending position	*/
	DLL_BAD_DATA,		/* Bad data pointer		*/
	DLL_BAD_CMPFUNC,	/* Bad comparison funct	*/
	DLL_EMPTY_LIST,		/* List is empty		*/
	DLL_LIST_NOT_EMPTY	/* List is not empty	*/
	} dllStat;

/******************* DLL node *******************/
typedef struct _dllLink {
	void			*data;	/* User data		*/
	struct _dllLink	*next;	/* Next DLL node	*/
	struct _dllLink	*prev;	/* Previous DLL node*/
	}	dllLink;

/*************** DLL control block **************/
typedef struct	{
	dllLink		*head;		/* First DLL node	*/
	dllLink		*curr;		/* Current DLL node	*/
	}	List;

/************** Function prototypes *************/
List    *dllCreateList(void);
dllStat  dllKillList(List *list);
dllStat  dllAdd(List *list,int dir,void *data);
void    *dllDelete(List *list,Position direction);
void    *dllGetData(List *list);
Position dllSetPos(List *list,Position dir);
dllStat  dllSearch(List *list,Position begin,
			Position stop,Position dir,
			int (*cmp)(void *,void *),void *udata);
