/************************************************
* MTCOPY.C - demonstrate multithread programming*
* with a simple file copy program.              *
************************************************/

#define INCL_DOS
#include <os2.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <process.h>

#define MAXSEGSIZE      65535u
#define THREADSTACKSIZE 1024

typedef struct segment
    {
    ULONG   segSem;     // semaphore
    SEL     segSel;     // selector
    USHORT  segfull;    // empty/full flag
    USHORT  segamount;  // # of bytes read
    } SEG;
typedef SEG *PSEG;

SEG seg;

HFILE input,output;
CHAR ReadThreadStack[THREADSTACKSIZE],
     WriteThreadStack[THREADSTACKSIZE];

void read_thread(void far *);
void write_thread(void far *);
void error_exit(USHORT errcode, char *fmt,...);
void mtcopy(char *source, char *dest);

void read_thread(void far *junk)
   {
   USHORT end_thread = FALSE;

   while(!end_thread)
      {
      DosSemRequest(&seg.segSem,SEM_INDEFINITE_WAIT);

      if(!seg.segfull)
         {
         USHORT errcode;

         if(errcode = DosRead(input,
                 MAKEP(seg.segSel,0),MAXSEGSIZE,
                       &seg.segamount))
             error_exit(errcode,"Error %u reading"
                        " file\n",errcode);

         if(!seg.segamount)     // nothing read:
            {                   // :End of file
            DosClose(input);    // close input
            end_thread = TRUE;  // terminate thread
            }
         seg.segfull = TRUE;    // for write & main
         }                      // threads
      DosSemClear(&seg.segSem);
      }
   _endthread();
   }

void write_thread(void far *junk)
   {
   USHORT end_thread = FALSE;

   while(!end_thread)
      {
      DosSemRequest(&seg.segSem,SEM_INDEFINITE_WAIT);
      if(seg.segfull)
         {
         USHORT errcode;

         if(!seg.segamount)     // EndOfFile reached
            {
            DosClose(output);   // close output
            end_thread = TRUE;  // terminate thread
            }
         else
            {
            if(errcode = DosWrite(output,
                   MAKEP(seg.segSel,0),
                   seg.segamount,&seg.segamount))
               error_exit(errcode,"Error %u writing"
                        " file\n",errcode);
            seg.segfull = FALSE;
            }
         }
      DosSemClear(&seg.segSem);
      }
   _endthread();
   }

void error_exit(USHORT errcode, char *fmt,...)
    {
    va_list argptr;

    va_start(argptr,fmt);
    vprintf(fmt,argptr);
    va_end(argptr);

    DosExit(errcode,EXIT_PROCESS);
    }

void mtcopy(char *source, char *dest)
    {
    USHORT errcode,action;

    memset(&seg,0,sizeof(seg));

    if(errcode = DosOpen(source,&input,&action,
            0L,0,FILE_OPEN,OPEN_ACCESS_READONLY |
            OPEN_SHARE_DENYWRITE,0L))
        error_exit(errcode,"Error %u opening %s\n",
                    errcode,source);

    if(errcode = DosOpen(dest,&output,&action,0L,
            FILE_NORMAL,FILE_CREATE|FILE_TRUNCATE,
            OPEN_ACCESS_WRITEONLY |
            OPEN_SHARE_DENYWRITE, 0L))
        error_exit(errcode,"Error %u opening/"
                "creating %s\n",errcode,dest);

    if(errcode = DosAllocSeg(MAXSEGSIZE,&seg.segSel,
                SEG_NONSHARED))
        error_exit(errcode,"Error %u allocating"
                " buffer\n",errcode);

    if(_beginthread(read_thread,ReadThreadStack,
            sizeof(ReadThreadStack),NULL) == -1)
       error_exit(0,"ERROR: start read thread\n");

    if(_beginthread(write_thread,WriteThreadStack,
            sizeof(WriteThreadStack),NULL) == -1)
        error_exit(0,"ERROR: start write thread\n");
    }

void main(int argc, char **argv)
    {
    if(argc != 3)
        error_exit(0,"Usage: MTCOPY <sourcefile>"
                    " <destinationfile>\n");

    mtcopy(argv[1],argv[2]);

    // substitute some other part of an application
    // here.  Copy is done by other threads

    while(TRUE)
        {
        DosSleep(1000L);
        DosSemRequest(&seg.segSem,
                        SEM_INDEFINITE_WAIT);
        if(seg.segfull && !seg.segamount)
            DosExit(0,EXIT_PROCESS);
        DosSemClear(&seg.segSem);
        }
    }
